/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2008  Evidence Srl
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation,
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

/*
 * Author: 2006 Paolo Gai
 * CVS: $Id: code.c,v 1.2 2008/07/17 18:59:32 nino Exp $
 */

/*
 * Epuck calibration & simple breitenberg algorithm implementation under Erika Enterprise
 *
 * Author: Javier Gomez Moreno
 *
 * USAGE:
 *
 * Donwload the hyperterminal client from e-puck developer team webpage.
 *
 * Add the e-puck to your computer list of bluetooth devices
 *
 * Connect with the hyperterminal to the COM port binded to you bluetooth device
 *
 * Once connected to the epuck, type h + enter to display the option in the hyperterminal.
 *
 * */

#define MS_PER_TICK 4.4
#define S_PER_TICK 0.0044
#define TRUE 1
#define FALSE 0
#define PI 3.14159265358979
#define RAD2DEG 57.29577951308232
#define MIN_SPEED 200
#define MAX_SPEED 500

#define PROXSCALING_FOLLOW 20
#define PROXSCALING_SHOCK 4
#define BASICSPEED 200
#define DETECTION_DISTANCE 30

#include "p30f6014A.h"

#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"

#include "e_epuck_ports.h"
#include "e_init_port.h"
#include "e_motors.h"
#include "math.h"
#include "e_uart_char.h"
//#include "e_bluetooth.h"
#include "e_ad_conv.h"
#include "e_prox_custom.h"

#include <string.h>
#include <stdio.h>
#include <ctype.h>

#ifndef _SEND_DATA_INLINE_FUNCTIONS
#define _SEND_DATA_INLINE_FUNCTIONS

#define uart_send_static_text(msg) do { e_send_uart1_char(msg,sizeof(msg)-1); while(e_uart1_sending()); } while(0)
#define uart_send_text(msg) do { e_send_uart1_char(msg,strlen(msg)); while(e_uart1_sending()); } while(0)

#define debug_send_static_text(msg) do { e_send_uart2_char(msg,sizeof(msg)-1); while(e_uart2_sending()); } while(0)
#define debug_send_text(msg) do { e_send_uart2_char(msg,strlen(msg)); while(e_uart2_sending()); } while(0)

#endif

//ODOMETRY RELATED VARIABLES

float increments_per_tour = 1000;
float wheel_diameter_left = 41;
float wheel_diameter_right = 41;

float axis_wheel_ratio = 1.288;
float scaling_factor = 1;

float b = 51.9/2; 	//Axis length/2 in mm
float e = 27.1; 	//Decentralized point in mm

float wheel_radius_left = 20.5;// = wheel_diameter_left/2;
float wheel_radius_right = 20.5;// = wheel_diameter_right/2;

int steps_l, steps_r, prev_steps_l = 0, prev_steps_r = 0;

float theta = PI * 0;

float linear_speed;
float angular_speed;
float pulse2rad = 2*PI/1000;

int steps = 1000;
int turns = 1;

float milimeters = 1000;

int square_iterations = 0;
int line_iterations = 0;
int increment = 8;
int increment_line = 10;

float xref = 0, yref = 0;

float ecos_theta=0.0;
float bsin_theta=0.0;
float bcos_theta=0.0;
float esin_theta=0.0;

float Ke=10.5;	//K for the engine control

float prev_xref = 0, prev_yref = 0;
float xspeed_ref, yspeed_ref;

float ang_speed_l, ang_speed_r, ang_speed_robot;
float lin_speed_l, lin_speed_r, lin_speed_robot;

float Ts = (12*S_PER_TICK);

float Kp = 5;
float Kv = 0.5;

float xc = 0, yc = 0;
float left_wheel_speed_ref, right_wheel_speed_ref;
float wref_l, wref_r, werr_l, werr_r;

float control_action_l = 0, control_action_r = 0;
float x = 0, y = 0;

//Bluetooth related vars

static char message[80];
int scanned = FALSE;
int pos_hal = 0;
int num_devices = 0;
int found = FALSE;
int intentos_fallidos = 0;
int synchro = FALSE;

//Seeking related variables (Breitenberg)

float theta_weigths[8] = {(-1.0/9.0)*PI, (-5.0/18.0)*PI, (-1.0/2.0)*PI, (-5.0/6.0)*PI, (5.0/6.0)*PI, (1.0/2.0)*PI, (5.0/18.0)*PI, (1.0/9.0)*PI};

int s, m;
long potential[2];
int speed[2];

int matrix_prox[2][8] =
	{{8,4,2,0,0,-4,-8,-16},
	{-16,-8,-4,0,0,2,4,8}};


void e_set_speed(int l, int r) {
	e_set_speed_left(l);
	e_set_speed_right(r);
}

int i;

/* Program the Timer1 peripheral to raise interrupts */
void T1_program(void)
{
	T1CON = 0x0100;        /* Stops the Timer1 and reset control reg	*/
	TMR1 = 0;              /* Clear contents of the timer register		*/
	PR1 = 0xffff;//;       /* Load the Period register wit the value 0xffff	*/
	IPC0bits.T1IP = 5;     /* Set Timer 1 priority to 5		*/
	IFS0bits.T1IF = 0;     /* Clear the Timer1 interrupt status flag	*/
	IEC0bits.T1IE = 1;     /* Enable Timer1 interrupts		*/
	T1CONbits.TON = 1;     /* Start Timer1 with prescaler settings at 1:1
							* and clock source set to the internal
							* instruction cycle			*/
/*	  T5CON = 0;                    //
	  T5CONbits.TCKPS=3;            // prescsaler = 256
	  TMR5 = 0;                     // clear timer 5
	  PR5 = (500.0*MILLISEC)/256.0; // interrupt every 500ms with 256 prescaler
	  IFS1bits.T5IF = 0;            // clear interrupt flag
	  IEC1bits.T5IE = 1;            // set interrupt enable bit
	  T5CONbits.TON = 1;            // start Timer5
*/
}

/* Clear the Timer1 interrupt status flag */
void T1_clear(void)
{
	IFS0bits.T1IF = 0;
//	IFS1bits.T5IF=0;
}

/* This is an ISR Type 2 which is attached to the Timer 1 peripheral IRQ pin
 * The ISR simply calls CounterTick to implement the timing reference
 */
ISR2(_T1Interrupt)
{
	/* clear the interrupt source */
	T1_clear();

	/* count the interrupts, waking up expired alarms */
	CounterTick(myCounter);
}

/*
 * This task just makes a led blink
 * Can be used with debugging purposes or just to test the planning
 */

TASK(TaskLED)
{
	LED0 = !LED0;
}

/*
 * This task implements the breitenberg algorithm for obstacle avoidance.
 * The e-puck will move on a straight line till it will find an obstacle,
 * then he will set a new course and keep on going on a straight line
 */

TASK (TaskSeek) {

	int irsvalue[8];
	int max_s = 0;
	int obstacle_found = FALSE;
	float obstacle_theta = 0;
	float obstacle_x = 0;
	float obstacle_y = 0;
	float divisor = 1;

	//uart_send_static_text("\r\n");
	//uart_send_static_text("Starting new iteration");

	//Calculation of the steps increment in each wheel
	steps_l = e_get_steps_left();
	steps_r = e_get_steps_right();
	//sprintf(message, "Steps Increment-> L=%d R=%d\r\n", steps_l, steps_r);
	//uart_send_text(message);

	//Calculation of the angular speed in each wheel
	ang_speed_l = pulse2rad*(steps_l - prev_steps_l)/Ts;
	ang_speed_r = pulse2rad*(steps_r - prev_steps_r)/Ts;
	//sprintf(message, "Angular Speed-> L=%f R=%f\r\n", (double)ang_speed_l, (double)ang_speed_r);
	//uart_send_text(message);

	//Calculation of the linear speed in each wheel
	lin_speed_l = ang_speed_l*wheel_radius_left;
	lin_speed_r = ang_speed_r*wheel_radius_right;
	//sprintf(message, "Linear Speed-> L=%f R=%f\r\n", (double)lin_speed_l, (double)lin_speed_r);
	//uart_send_text(message);

	//Calculation of the total linear and angular speed of the robot
	lin_speed_robot = (lin_speed_l + lin_speed_r)/2;
	ang_speed_robot = (lin_speed_r - lin_speed_l)/(2*b);
	//sprintf(message, "Robot Linear Speed-> %f\r\n", lin_speed_robot);
	//uart_send_text(message);
	//sprintf(message, "Robot Angular Speed-> %f\r\n", ang_speed_robot);
	//uart_send_text(message);

	//Calculation of the position and the orientation of the robot
	x=x+lin_speed_robot*Ts*cosf(theta);
	y=y+lin_speed_robot*Ts*sinf(theta);
	theta = theta+ang_speed_robot*Ts;
	//sprintf(message, "Position-> X=%f Y=%f theta=%f \r\n", (double)x, (double)y,(double)theta);
	//uart_send_text(message);

	prev_steps_l = steps_l;
	prev_steps_r = steps_r;

	for (s = 0; s < 8; s++) {
		irsvalue[s] = e_get_calibrated_prox(s);
		if (irsvalue[s] < 50)	//if no significative value is obtained, avoid random roaming.
			irsvalue[s] = 0;
		//potential[m] += (matrix_prox[m][s]*e_get_calibrated_prox(s)); // get values from proximity sensors


		if(irsvalue[s]>irsvalue[max_s])
			max_s = s;

		if(irsvalue[s] > 50)
			obstacle_found = TRUE;

	}
	//sprintf(message, "0:%d 1:%d 2:%d 3:%d 4:%d 5:%d 6:%d 7:%d\tSL:%d SR:%d\r\n", irsvalue[0], irsvalue[1], irsvalue[2], irsvalue[3], irsvalue[4], irsvalue[5], irsvalue[6], irsvalue[7], speed[1], speed[0]);
	//sprintf(message, "0:%d 1:%d 2:%d 3:%d 4:%d 5:%d 6:%d 7:%d\r\n", irsvalue[0], irsvalue[1], irsvalue[2], irsvalue[3], irsvalue[4], irsvalue[5], irsvalue[6], irsvalue[7]);
	//uart_send_static_text(message);
	for (m = 0; m < 2; m++)
	{
		potential[m] = 0;
		for (s = 0; s < 8; s++) {
			potential[m] += (matrix_prox[m][s]*irsvalue[s]); // get values from proximity sensors
		}
		speed[m] = (potential[m]/PROXSCALING_SHOCK + BASICSPEED);
	}

	//sprintf(message, "SL:%d SR:%d\r\n", speed[1], speed[0]);
	//uart_send_text(message);

	if (obstacle_found) {
		obstacle_theta =(float)(irsvalue[(max_s+7)%8] * theta_weigths[(max_s-1)%8] + irsvalue[max_s] * theta_weigths[max_s] + irsvalue[(max_s+1)%8] * theta_weigths[(max_s+1)%8]);
		divisor = (float)(irsvalue[(max_s+7)%8] + irsvalue[max_s] + irsvalue[(max_s+1)%8]);
		//sprintf(message,"Divisor = %f\r\n",(double)divisor);
		//uart_send_text(message);
		if (divisor == 0) {
			uart_send_static_text("Divisor == 0!!! Aborting calculations\r\n");
		}
		if (divisor!=0) {
			obstacle_theta = obstacle_theta/divisor;
			//obstacle_theta = obstacle_theta * RAD2DEG;
			//sprintf(message,"Obstacle theta/divisor = %f\r\n",(double)obstacle_theta);
			//uart_send_text(message);

			obstacle_theta += theta;

			//sprintf(message,"Obstacle theta += theta = %f\r\n",(double)obstacle_theta);
			//uart_send_text(message);

			//uart_send_static_text("X calculation\r\n");
			obstacle_x = x + DETECTION_DISTANCE * cosf(obstacle_theta);
			//sprintf(message,"%f = %f + 40 * %f\r\n",(double)obstacle_x, (double)x, (double)cosf(obstacle_theta));
			//uart_send_text(message);

			//uart_send_static_text("Y calculation\r\n");
			obstacle_y = y + DETECTION_DISTANCE * sinf(obstacle_theta);
			//sprintf(message,"%f = %f + 40 * %f\r\n",(double)obstacle_y, (double)y, (double)sinf(obstacle_theta));
			//uart_send_text(message);

			//sprintf(message,"pos X = %f Y = %f Th = %f\r\n", (double)x ,(double)y, (double)theta);
			//uart_send_text(message);
			//sprintf(message,"obs X = %f Y = %f Th = %f\r\n", (double)obstacle_x, (double)obstacle_y, (double)obstacle_theta);
			//uart_send_text(message);
		}
		obstacle_found = FALSE;
	}

	if((speed[1] < 50 && speed[1] > -50)
			&& (speed[0] < 50 && speed[0] > -50)) {
		speed[1] = speed[1] * 20;
		speed[0] = speed[0] * 20;
	}

	//Control action saturation
	if (speed[1] > 1000)
		speed[1] = 1000;
	else if (speed[1] < -1000 )
		speed[1] = -1000;

	if (speed[0] > 1000)
		speed[0] = 1000;
	else if (speed[0] < -1000 )
		speed[0] = -1000;

	//sprintf(message, "SL:%d SR:%d \r\n", speed[1], speed[0]);
	//uart_send_text(message);

	e_set_speed_left(speed[1]);
	e_set_speed_right(speed[0]);

	//sprintf(message, "Finishing iteration \r\n");
	//uart_send_text(message);

}

/*
 * This task generates a squared trajectory and make the e-puck follow it.
 *
 * It is useful to calibrate the wheel diameters. Run the task, if the e-puck does not close the
 * square then increase the left wheel diameter and increase the right wheel diameter the
 * same amount of units (diameterLeft = diameterLeft + inc AND diameterRight = diameter Right - inc)
 *
 * If on the contrary the trajectory is too closed, decrease the left diameter and increase the right
 * diameter.
 *
 * This can be done through the hyperterminal interface or changing the default value of the variables
 */

TASK (TaskSquareDP) {


	if(square_iterations < 600) {
		//Generation of the square trajectory
		if ((square_iterations >= 0) && (square_iterations < 101)) {
			xref += increment;
		} else if ((square_iterations > 100) && (square_iterations < 201)) {
			yref += increment;
		} else if ((square_iterations > 200) && (square_iterations < 301)) {
			xref -= increment;
		} else if ((square_iterations > 300) && (square_iterations < 401)) {
			yref -= increment;
		} else if ((square_iterations > 400) && (square_iterations < 405)) {
			yref -= 7;
		}
		sprintf(message, "Iteration-> %d\r\n", square_iterations);
		uart_send_text(message);
		sprintf(message, "Reference-> X=%f Y=%f\r\n", (double)xref, (double)yref);
		uart_send_text(message);

		//Calculation of the speed reference
		xspeed_ref = (xref - prev_xref)/Ts;
		yspeed_ref = (yref - prev_yref)/Ts;

		sprintf(message, "Speed Reference-> X=%f Y=%f\r\n", (double)xspeed_ref, (double)yspeed_ref);
		uart_send_text(message);

		//Calculation of the steps increment in each wheel
		steps_l = e_get_steps_left();
		steps_r = e_get_steps_right();
		sprintf(message, "Steps Increment-> L=%d R=%d\r\n", steps_l, steps_r);
		uart_send_text(message);

		//Calculation of the angular speed in each wheel
		ang_speed_l = pulse2rad*(steps_l - prev_steps_l)/Ts;
		ang_speed_r = pulse2rad*(steps_r - prev_steps_r)/Ts;
		sprintf(message, "Angular Speed-> L=%f R=%f\r\n", (double)ang_speed_l, (double)ang_speed_r);
		uart_send_text(message);

		//Calculation of the linear speed in each wheel
		lin_speed_l = ang_speed_l*wheel_radius_left;
		lin_speed_r = ang_speed_r*wheel_radius_right;
		sprintf(message, "Linear Speed-> L=%f R=%f\r\n", (double)lin_speed_l, (double)lin_speed_r);
		uart_send_text(message);

		//Calculation of the total linear and angular speed of the robot
		lin_speed_robot = (lin_speed_l + lin_speed_r)/2;
		ang_speed_robot = (lin_speed_r - lin_speed_l)/(2*b);
		sprintf(message, "Robot Linear Speed-> %f\r\n", (double)lin_speed_robot);
		uart_send_text(message);
		sprintf(message, "Robot Angular Speed-> %f\r\n", (double)ang_speed_robot);
		uart_send_text(message);


		//Calculation of the position and the orientation of the robot
		x=x+lin_speed_robot*Ts*cosf(theta);
		y=y+lin_speed_robot*Ts*sinf(theta);
		theta = theta+ang_speed_robot*Ts;
		sprintf(message, "Position-> X=%f Y=%f theta=%f \r\n", (double)x, (double)y, (double)theta);
		uart_send_text(message);

		sprintf(message, "%f\t%f\t%f\t%f\t%f\r\n", (double)x, (double)y , (double)theta, (double)xref, (double)yref);
		debug_send_text(message);

		//Auxiliary Calculations
		ecos_theta = e*cosf(theta);
		esin_theta = e*sinf(theta);
		bcos_theta = b*cosf(theta);
		bsin_theta = b*sinf(theta);

		//Calculation of the kinematic control of the robot
		xc = Kv*xspeed_ref+Kp*(xref-(x+ecos_theta));
		yc = Kv*yspeed_ref+Kp*(yref-(y+esin_theta));
		sprintf(message, "Position KC-> X=%f Y=%f \r\n", (double)xc, (double)yc);
		uart_send_text(message);

		//Calculation of the robot inverse kinematics
		left_wheel_speed_ref = ((ecos_theta+bsin_theta)*xc+(esin_theta-bcos_theta)*yc)/e;
		right_wheel_speed_ref = ((ecos_theta-bsin_theta)*xc+(esin_theta+bcos_theta)*yc)/e;
		sprintf(message, "Speed references IK-> L=%f R=%f \r\n", (double)left_wheel_speed_ref, (double)right_wheel_speed_ref);
		uart_send_text(message);

		//Calculation of the angular speed references for the dynamic control
		wref_l = left_wheel_speed_ref/wheel_radius_left;
		wref_r = right_wheel_speed_ref/wheel_radius_right;
		sprintf(message, "Angular Speed references IK-> L=%f R=%f \r\n", (double)wref_l, (double)wref_r);
		uart_send_text(message);

		//Calculation of the angular speed error
		werr_l = wref_l-ang_speed_l;
		werr_r = wref_r-ang_speed_r;
		sprintf(message, "Angular Speed error IK-> L=%f R=%f \r\n", (double)werr_l, (double)werr_r);
		uart_send_text(message);

		//Calculation of the control actions to perform in each wheel
		control_action_l = werr_l*Ke;
		control_action_r = werr_r*Ke;

		//Saturation of control actions
		if (control_action_l > 1000)
			control_action_l = 1000;
		if (control_action_l < -1000)
			control_action_l = -1000;
		if (control_action_r > 1000)
			control_action_r = 1000;
		if (control_action_r < -1000)
			control_action_r = -1000;

		//Application of the control action
		e_set_speed(control_action_l,control_action_r);

		sprintf(message, "Control Action-> L=%f R=%f\r\n", (double)control_action_l, (double)control_action_r);
		uart_send_text(message);

		uart_send_static_text("\r\n");

		prev_xref = xref;
		prev_yref = yref;

		prev_steps_l = steps_l;
		prev_steps_r = steps_r;

	} else {
		//end the task
		uart_send_static_text("Test finished, launching main task\r\n");
		ActivateTask(TaskCalibrateWheels);

		uart_send_static_text("Square task finished, shutting down...\r\n");
		CancelAlarm(AlarmSquareDP);

	}

	square_iterations++;
}

/*
 * This task generates a straight trajectory and make the e-puck follow it.
 *
 * It is useful to calibrate the scaling factor. Run the task, if the e-puck does not move the same
 * amount of unit in the real life than programmed then calculate scaling factor as follows:
 *
 * scalingFactor = realDistance/parameterDistance
 *
 * This can be done through the hyperterminal interface or changing the default value of the variables.
 */

TASK (TaskLineDP) {

	if(line_iterations < 150) {
		//Generation of the square trajectory
		if ((line_iterations >= 0) && (line_iterations < 100))
			xref += increment_line;
		else {
			xref += increment_line;
			if (xref > (1000+e))
				xref = 1000+e;
		}

		//sprintf(message, "Iteration-> %d\r\n", line_iterations);
		//uart_send_text(message);
		//sprintf(message, "Reference-> X=%f Y=%f\r\n", (double)xref, (double)yref);
		//uart_send_text(message);

		//Calculation of the speed reference
		xspeed_ref = (xref - prev_xref)/Ts;
		yspeed_ref = (yref - prev_yref)/Ts;

		//sprintf(message, "Speed Reference-> X=%f Y=%f\r\n", (double)xspeed_ref, (double)yspeed_ref);
		//uart_send_text(message);

		//Calculation of the steps increment in each wheel
		steps_l = e_get_steps_left();
		steps_r = e_get_steps_right();
		//sprintf(message, "Steps Increment-> L=%d R=%d\r\n", steps_l, steps_r);
		//uart_send_text(message);

		//Calculation of the angular speed in each wheel
		ang_speed_l = pulse2rad*(steps_l - prev_steps_l)/Ts;
		ang_speed_r = pulse2rad*(steps_r - prev_steps_r)/Ts;
		//sprintf(message, "Angular Speed-> L=%f R=%f\r\n", (double)ang_speed_l, (double)ang_speed_r);
		//uart_send_text(message);

		//Calculation of the linear speed in each wheel
		lin_speed_l = ang_speed_l*wheel_radius_left;
		lin_speed_r = ang_speed_r*wheel_radius_right;
		//sprintf(message, "Linear Speed-> L=%f R=%f\r\n", (double)lin_speed_l, (double)lin_speed_r);
		//uart_send_text(message);

		//Calculation of the total linear and angular speed of the robot
		lin_speed_robot = (lin_speed_l + lin_speed_r)/2;
		ang_speed_robot = (lin_speed_r - lin_speed_l)/(2*b);
		//sprintf(message, "Robot Linear Speed-> %f\r\n", (double)lin_speed_robot);
		//uart_send_text(message);
		//sprintf(message, "Robot Angular Speed-> %f\r\n", (double)ang_speed_robot);
		//uart_send_text(message);


		//Calculation of the position and the orientation of the robot
		x=x+lin_speed_robot*Ts*cosf(theta);
		y=y+lin_speed_robot*Ts*sinf(theta);
		theta = theta+ang_speed_robot*Ts;
		//sprintf(message, "Position-> X=%f Y=%f theta=%f \r\n", (double)x, (double)y, (double)theta);
		//uart_send_text(message);

		//sprintf(message, "%f\t%f\t%f\t%f\t%f\r\n", (double)x, (double)y , (double)theta, (double)xref, (double)yref);
		//debug_send_text(message);

		//Auxiliary Calculations
		ecos_theta = e*cosf(theta);
		esin_theta = e*sinf(theta);
		bcos_theta = b*cosf(theta);
		bsin_theta = b*sinf(theta);

		//Calculation of the kinematic control of the robot
		xc = Kv*xspeed_ref+Kp*(xref-(x+ecos_theta));
		yc = Kv*yspeed_ref+Kp*(yref-(y+esin_theta));
		//sprintf(message, "Position KC-> X=%f Y=%f \r\n", (double)xc, (double)yc);
		//uart_send_text(message);

		//Calculation of the robot inverse kinematics
		left_wheel_speed_ref = ((ecos_theta+bsin_theta)*xc+(esin_theta-bcos_theta)*yc)/e;
		right_wheel_speed_ref = ((ecos_theta-bsin_theta)*xc+(esin_theta+bcos_theta)*yc)/e;
		//sprintf(message, "Speed references IK-> L=%f R=%f \r\n", (double)left_wheel_speed_ref, (double)right_wheel_speed_ref);
		//uart_send_text(message);

		//Calculation of the angular speed references for the dynamic control
		wref_l = left_wheel_speed_ref/wheel_radius_left;
		wref_r = right_wheel_speed_ref/wheel_radius_right;
		//sprintf(message, "Angular Speed references IK-> L=%f R=%f \r\n", (double)wref_l, (double)wref_r);
		//uart_send_text(message);

		//Calculation of the angular speed error
		werr_l = wref_l-ang_speed_l;
		werr_r = wref_r-ang_speed_r;
		//sprintf(message, "Angular Speed error IK-> L=%f R=%f \r\n", (double)werr_l, (double)werr_r);
		//uart_send_text(message);

		//Calculation of the control actions to perform in each wheel
		control_action_l = werr_l*Ke;
		control_action_r = werr_r*Ke;

		//Saturation of control actions
		if (control_action_l > 1000)
			control_action_l = 1000;
		if (control_action_l < -1000)
			control_action_l = -1000;
		if (control_action_r > 1000)
			control_action_r = 1000;
		if (control_action_r < -1000)
			control_action_r = -1000;

		//Application of the control action
		e_set_speed(control_action_l,control_action_r);

		//sprintf(message, "Control Action-> L=%f R=%f\r\n", (double)control_action_l, (double)control_action_r);
		//uart_send_text(message);

		//uart_send_static_text("\r\n");

		prev_xref = xref;
		prev_yref = yref;

		prev_steps_l = steps_l;
		prev_steps_r = steps_r;

	} else {
		//end the task
		uart_send_static_text("Test finished, launching main task\r\n");
		ActivateTask(TaskCalibrateWheels);

		uart_send_static_text("Line task finished, shutting down...\r\n");
		CancelAlarm(AlarmLineDP);

	}

	line_iterations++;
}

/*
 * This task supplies a hyperterminal interface that allows you to run three test used for robot calibration.
 * Also allows you to run the Breitenberg algorithm but this last one will lock the execution on itself so to get
 * back access to the interface you MUST restart the e-puck from the blue button present on top of the robot.
 *
 * Further explanation of the tests can be found in the following comments.
 */

TASK (TaskCalibrateWheels) {

	int s = 0;
	char c = 'h';
	int index = 1;

	while (c!='E') {
		while(c=='\n' || c=='\r') {
			e_getchar_uart1(&c);
		}
		message[0]=c;
		index = 1;
		do if (e_getchar_uart1(&c))
			message[index++]=c;
		while (c!='\n' && c!='\r');
		message[index++] = '\0';
		message[0]=toupper(message[0]);

		switch (message[0]) {

		/*
		 * Test one: Increments per tour.
		 *
		 * Run for a value of 1000 (e-puck documentation indicates this value as the number of steps per tour).
		 * Check that the wheel has done exactly one revolution by putting a marker on the wheel. If is not
		 * the case then look for a value such that the wheel will do a perfect turn. Then check it by doing
		 * 10 turns in a row instead of one.
		 */
								case '1':
									sscanf(message,"1,%d\r",&steps);
									sprintf(message,"Testing rotation for: %d \r\n", steps);
									uart_send_text(message);

									e_set_steps_left(0);
									e_set_steps_right(0);

									// linear speed increase
									while (e_get_steps_left() < 100 || e_get_steps_right() < 100) {
										s = (int)(MAX_SPEED * (e_get_steps_left() + e_get_steps_right()) / 200);
										s = s < MIN_SPEED ? MIN_SPEED : s;
										s = s > MAX_SPEED ? MAX_SPEED : s;
										e_set_speed(s,s);
										sprintf(message,"Speed = %d steps/s\r\n", s);
										uart_send_text(message);
									}

									// max speed
									while (e_get_steps_left() < steps-400 || e_get_steps_right() < steps-400) {
										e_set_speed(MAX_SPEED,MAX_SPEED);
										sprintf(message,"Speed = %d steps/s\r\n", MAX_SPEED);
										uart_send_text(message);
									}

									//linear speed decrease over 400 increments
									while (e_get_steps_left() < steps || e_get_steps_right() < steps) {
										s = (int)(MAX_SPEED * (steps - (e_get_steps_left() + e_get_steps_right()) / 2) / 400);
										s = s < MIN_SPEED ? MIN_SPEED : s;
										s = s > MAX_SPEED ? MAX_SPEED : s;
										e_set_speed(s,s);
										sprintf(message,"Speed = %d steps/s\r\n", s);
										uart_send_text(message);

									}
									e_set_speed(0,0);
									uart_send_static_text("Increments test Finished\r\n");
									break;

		/*
		 * Test two: Axis wheel ratio
		 *
		 * If you search in the documentation of the e-puck you will find that the wheel diameter should be 41mm for each wheel
		 * and that the distance between the wheels is 53mm which gives a ratio of 1.293. To verify this we will command the
		 * robot to rotate in place several times. To do a complete turn we will need 1000 * 1.293 = 1293 increments. The number
		 * of steps can be adjusted.
		 * Run for a value of 1293 and adjust the value as before to have one perfect turn. Then verify it with 10 turns or more.
		 */

								case '2': // test 2: Axis wheel ratio
									sscanf(message,"2,%f,%d\r",&axis_wheel_ratio,&turns);
									sprintf(message,"Testing axis wheel ratio for: %f and %d turns\r\n", (double)axis_wheel_ratio,turns);
									uart_send_text(message);

									e_set_steps_left(0);
									e_set_steps_right(0);

									while (e_get_steps_left() > -100 || e_get_steps_right() < 100)
									  {
									    s = (int)(MAX_SPEED * (-e_get_steps_left() + e_get_steps_right()) / 200);
									    s = s < MIN_SPEED ? MIN_SPEED : s;
									    s = s > MAX_SPEED ? MAX_SPEED : s;
									    e_set_speed(-s,s);
									    sprintf(message,"SL = %d SR = %d steps/s\r\n", -s, s);
									    uart_send_text(message);
									  }

									  while (e_get_steps_left() > -(axis_wheel_ratio*1000*turns)+400
									  && e_get_steps_right() < (axis_wheel_ratio*1000*turns)-400)
									  {
									    e_set_speed(-MAX_SPEED,MAX_SPEED);
									    sprintf(message,"SL = %d SR = %d steps/s\r\n", -MAX_SPEED, MAX_SPEED);
									    uart_send_text(message);
									  }

									  // linear speed decrease over 400 increments
									  while (e_get_steps_left() > -axis_wheel_ratio*1000*turns  || e_get_steps_right() < axis_wheel_ratio*1000*turns)
									  {
									    s = (int)(MAX_SPEED * (axis_wheel_ratio*1000*turns - (-e_get_steps_left() + e_get_steps_right()) / 2) / 400);
									    s = s < MIN_SPEED ? MIN_SPEED : s;
									    s = s > MAX_SPEED ? MAX_SPEED : s;
									    e_set_speed(-s,s);
									    sprintf(message,"SL = %d SR = %d steps/s\r\n", -s, s);
									    uart_send_text(message);
									  }
									  uart_send_static_text("Axis wheel ratio test Finished\r\n");
									  e_set_speed(0,0);
									break;

		/*
		 * Test three: Wheel diameters
		 *
		 * We will give 4 waypoints to the robots such that it will move along a square. By default the side of the square
		 * measures 0.8m but if you have more space you can increase this size to get better results.
		 *
		 * The robot will move and come back near it's start position. If the square is not closed, as shown on the figure
		 * called "Open and closed square trajectories", increase the left diameter and decrease the right diameter.
		 * This means: diameterLeft = diameterLeft + inc and diameterRight = diameterRight − inc.
		 * If on the contrary the trajectory is too closed, as shown on the same figure, decrease the left diameter and
		 * increase the right diameter.
		 */

								case '3':
									sscanf(message,"3,%f,%f\r",&wheel_diameter_left, &wheel_diameter_right);
								    sprintf(message,"Testing wheel diameters for (%.2f, %.2f)\r\n", (double)wheel_diameter_left, (double)wheel_diameter_right);
								    uart_send_text(message);

								    //(Re)Calculating wheel radiuses
								    wheel_radius_left = wheel_diameter_left/2;
								    wheel_radius_right = wheel_diameter_right/2;

								    //Setting up the steps
								    e_set_steps_left(0);
								    e_set_steps_right(0);

								    //Reseting variable values
								    theta = PI * 0;
								   	x = 0;
								   	y = 0;
								   	prev_steps_l = 0;
								   	prev_steps_r = 0;
								   	prev_xref = 0;
								   	prev_yref = 0;
								   	xc = 0;
								   	yc = 0;
								   	square_iterations = 0;

								    //Launching the square trajectory test task
								    uart_send_static_text("Launching the square task\r\n");
								    SetRelAlarm(AlarmSquareDP, 10,10);

								    //Finishing the main task
								    uart_send_static_text("suspending main task...\r\n");
								    c = 'E';
									break;

		/*
		 * Test four: Scaling factor
		 *
		 * The scaling factor is the easiest to find out, we just run the robot on a straight line for a given distance
		 * (1m in the code) and measure the actual covered distance.
		 *
		 * scalingFactor = realDistance/parameterDistance
		 */

								case '4':
									sscanf(message,"4,%f", &milimeters);
								    sprintf(message,"Testing scalling factor for %f milimeters\r\n", (double)milimeters);
								    uart_send_text(message);

								    wheel_radius_left = wheel_diameter_left/2;
								    wheel_radius_right = wheel_diameter_right/2;
								    //Setting up the steps
								    e_set_steps_left(0);
								    e_set_steps_right(0);

								    //Reseting variable values
								    theta = PI * 0;
								    x = 0;
								    y = 0;
								    //xref = e;
								    xref = 0;
								    yref = 0;
								    prev_steps_l = 0;
								    prev_steps_r = 0;
								    prev_xref = 0;
								    prev_yref = 0;
								    xc = 0;
								    yc = 0;

								    line_iterations = 0;

								    //Launching the straight trajectory test task
								    uart_send_static_text("Launching the straight line task\r\n");
								    SetRelAlarm(AlarmLineDP, 10,10);

								    //Finishing the main task
								    uart_send_static_text("suspending main task...\r\n");

								    c = 'E';

								    break;

		/*
		 * Breitenberg Algorithm.
		 * This option WILL LOCK the execution on it. To get back the hyperterminal interface YOU MUST RESET the e-puck
		 */

								case '5':

									uart_send_static_text("Running Breitenberg...\r\n");

									SetRelAlarm(AlarmSeek, 100,12);

									//Finishing the main task
									c = 'E';

									break;

								case 'H': //Help
									uart_send_static_text("\r\n");
									uart_send_static_text("\"1,steps\"            Test Increment \r\n");
									uart_send_static_text("\"2,aw_ratio,n_turns\" Test Axis wheel ratio\r\n");
									uart_send_static_text("\"3,DL,DR\"            Test wheels diameters\r\n");
									uart_send_static_text("\"4,dist_in_mm\"       Test scale factor\r\n");
									uart_send_static_text("\"5\"       			  Breitenberg Algorithm\r\n");
									uart_send_static_text("\"H\"	              Help\r\n");
									uart_send_static_text("\"E\"	              Exit\r\n");
									break;

								case 'E':

									uart_send_static_text("Main task shutting down...\r\n");
									break;

								default:
									uart_send_static_text("Command not found\r\n");
									break;
								}

	}
}

int main(void)
{
	/* Program Timer 1 to raise interrupts */
	T1_program();
	e_init_port();
	e_init_motors();
	e_init_uart1();   // initialize UART to 115200 Kbaud
	e_init_uart2();
	e_set_steps_left(0);
	e_set_steps_right(0);

	char trash[4];

	LED0=0;
	LED1=0;
	LED2=0;
	LED3=0;

	LED4=0;
	LED5=0;
	LED6=0;
	LED7=0;

	i=0;

	debug_send_static_text("e-puck Initialized!\r\n");
	debug_send_static_text("Initializing AD_SCAN...\r\n");
	e_init_ad();
	e_stop_prox();
	e_init_prox();
	debug_send_static_text("Calibrating IR...\r\n");
	e_calibrate_ir();
	debug_send_static_text("Calibration complete!\r\n");
	debug_send_static_text("Cleaning uart buffer...\r\n");

	//We wait for the e-puck to stop sending data
	while(e_uart1_sending()) {};

	///Reset if Power on (some problem for few robots)
	if (RCONbits.POR) {
		RCONbits.POR=0;
		__asm__ volatile ("reset");
	}

	//Emptying the uart

	while(e_ischar_uart1()) {
		e_getchar_uart1(trash);
		trash[1]='\r';
		trash[2]='\n';
		trash[3]='\0';
		debug_send_static_text("Trash cleaned:\r\n");
		debug_send_text(trash);
	}
	debug_send_static_text("uart clean!\r\n");
	/* Program cyclic alarms which will fire after an offset of 10 counter
	 * ticks, and after that periodically every 5 ticks */
	debug_send_static_text("Launching tasks...\r\n");
	SetRelAlarm(AlarmLED, 10,  100);
	ActivateTask(TaskCalibrateWheels);
	debug_send_static_text("Complete!\r\n");



	/* Forever loop: background activities (if any) should go here */
	for (;;);
	return 0;
}
